package de.cnam.jemina.i18n;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * F&uuml;r die Internationalisierung
 * 
 * 
 * @author CNAM Team (Nadine, Mike, Jens)
 * @since 24.11.2012
 */
public class LocalizationManager {
    // Konstanten
    private static final String RESOURCE_FILE = "messageResource";

    // Variablen
    private final String classname;
    private ResourceBundle resourceBundle;

    // Ressourcen-Verwaltung:
    private static Map<Locale, ResourceBundle> resources;

    // Logging:
    private final Logger log = LoggerFactory.getLogger(LocalizationManager.class);


    // Static content:
    static {
        resources = new HashMap<>(5);
    }


    /**
     * Privater Konstruktor.
     * Um neue Instanzen dieser Klasse zu erzeugen bitte die Factory-Methode
     * {@link #getLocalizationManager(java.lang.Class)} verwenden.
     *
     * @param pvLocale Sprache in der die Instanz des LocalizationManager sein soll
     * @param pvClassname Name der Klasse in dem der LocalizationManager verwendet wird (dient als Pr&auml;fix)
     */
    private LocalizationManager(Locale pvLocale, String pvClassname) {
        super();

        // Setze Praefix:
        classname = pvClassname.toLowerCase();

        // Ist eine Ressource mit der Sprache bereits vorhanden?
        if (resources.containsKey(pvLocale)) {
            resourceBundle = resources.get(pvLocale);

            if (log.isDebugEnabled()) {
                log.debug("Konnte Sprache {} aus der Ressourcen-Verwaltung laden.", pvLocale.toString());
            }
        } else {
            // Ressource noch nicht geoeffnet
            if (log.isDebugEnabled()) {
                log.debug("Ressource fuer die Sprache {} wird geladen.", pvLocale.toString());
            }

            // Lade Ressource:
            try {
                resourceBundle = ResourceBundle.getBundle(RESOURCE_FILE);
                resources.put(pvLocale, resourceBundle);
            } catch (Exception ex) {
                log.error("Konnte Resource-Datei '{}' nicht laden!", RESOURCE_FILE, ex);
            }
        }
    }

    /**
     * Liefert eine Instanz des LocalizationManager f&uuml;r die
     * &uuml;bergebene Klasse.
     *
     * Die Sprache kann &uuml;ber {@link Locale#setDefault(java.util.Locale)} festgelegt werden.
     * @param pvClass Klasse in der der LocalizationManager verwendet wird
     * @return Eine Instanz der Klasse <i>LocalizationManager</i>
     */
    public static LocalizationManager getLocalizationManager(Class pvClass) {
        return getLocalizationManager(pvClass, Locale.getDefault());
    }

    /**
     * Liefert eine Instanz des LocalizationManager f&uuml;r die
     * &uuml;bergebene Klasse und Sprache.
     *
     * @param pvClass Klasse in der der LocalizationManager verwendet wird
     * @param pvLocale Sprache die verwendet werden soll
     * @return Eine Instanz der Klasse <i>LocalizationManager</i>
     */
    public static LocalizationManager getLocalizationManager(Class pvClass, Locale pvLocale) {
        return new LocalizationManager(pvLocale, pvClass == null ? "" : pvClass.getName());
    }


    /**
     * Sucht zu der &uuml;bergebenen ID die passende &Uuml;bersetzung.
     * @param pvStr Message-ID (diese wird unver&auml;ndert gepr&uuml;ft)
     * @return Liefert die &Uuml;bersetzung oder <b>null</b> wenn nichts gefunden wurde.
     */
    public String localize(String pvStr) {
        return localize(pvStr, null);
    }

    /**
     * Sucht zu der &uuml;bergebenen ID die passende &Uuml;bersetzung.
     *
     * Wird keine passende &Uuml;bersetzung gefunden wird der &uuml;bergebene <i>default</i>-Wert
     * zur&uuml;ckgegeben.
     *
     * Variablen werden durch den Inhalt des &uuml;bergebenen String-Arrays ersetzt.
     * Variablen werden in der .properties Datei mit geschweiften Klammern und einer Nummer
     * (startet bei 0) angegeben. Beispiel:
     *
     * <i>Willkommen, <b>{0}</b>! Heute ist dein {1}. Besuch!</i>
     *
     * @param pvStr
     * @param pvDefault
     * @param pvVar
     * @return
     */
    public String localize(String pvStr, String pvDefault, String[] pvVar) {
        try {
            MessageFormat lvMf = new MessageFormat(localize(pvStr, pvDefault));
            return lvMf.format(pvVar);
        } catch (Exception ex) {
            log.error("Es gabe einen Fehler bei der Uebersetzung von '{}'.", pvStr, ex);
            return pvDefault;
        }
    }

    /**
     * Sucht zu der &uuml;bergebenen ID die passende &Uuml;bersetzung.
     * Wird keine &Uuml;bersetzung gefunden wird der &uuml;bergebene <i>default</i>-Wert
     * zur&uuml;ckgegeben.
     * @param pvStr Message-ID (diese wird unver&auml;ndert gepr&uuml;ft)
     * @param pvDefault Wert der zur&uuml;ckgegeben werden soll, wenn keine &Uuml;bersetzung gefunden wurde
     * @return Liefert die &Uuml;bersetzung oder den <i>default</i>-Wert, welcher &uuml;bergeben wurde
     */
    public String localize(String pvStr, String pvDefault) {
        // Baue Key zusammen
        StringBuilder lvKeynameStrB = new StringBuilder();
        lvKeynameStrB.append(classname);
        lvKeynameStrB.append("_");
        if (pvStr != null) {
            pvStr = pvStr.replaceAll(" ", "");
            lvKeynameStrB.append(pvStr.toLowerCase());
        } else {
            lvKeynameStrB.append("null");
            log.warn("Der uebergebene String ist NULL.");
        }

        String lvKey = lvKeynameStrB.toString();

        // Logging
        if (log.isDebugEnabled()) {
            log.debug("Suche nach '{}'", lvKey);
        }

        // Ressource nach dem Key fragen:
        try {
            final String lvValue = resourceBundle.getString(lvKey);
            return lvValue == null ? pvDefault : lvValue;
        } catch (MissingResourceException mre) {
            log.warn("String '{}' wurde nicht gefunden! Sprache: {}", new Object[] { lvKey, resourceBundle.getLocale().toString() } );
        } catch (Exception ex) {
            log.error("Es gab einen Fehler beim bearbeiten des Strings '{}'", lvKey, ex);
        }

        // Gebe default-Wert zurueck
        return pvDefault;
    }

    /**
     * &Uuml;bersetzt den &uuml;bergebenen String.
     * Im Vergleich zu {@link #localize(java.lang.String)} wird hier kein Pr&auml;fix verwendet.
     * @param pvStr String der &uuml;bersetzt werden soll
     * @return Wird keine &Uuml;bersetzung gefunden, wird der String zur&uuml;ckgegeben
     */
    public String localizeStr(String pvStr) {
        try {
            final String lvValue = resourceBundle.getString(pvStr.toLowerCase());
            return lvValue == null ? pvStr : lvValue;
        } catch (MissingResourceException mre) {
            log.warn("String '{}' wurde nicht gefunden! Sprache: {}", new Object[] { pvStr, resourceBundle.getLocale().toString() } );
        } catch (Exception ex) {
            log.error("Es gab einen Fehler beim bearbeiten des Strings '{}'", pvStr, ex);
        }

        return pvStr;
    } 
}
