package de.cnam.jemina.ui.components;

import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Component;
import com.vaadin.ui.CustomComponent;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.TextArea;
import com.vaadin.ui.TextField;
import com.vaadin.ui.VerticalLayout;
import de.cnam.jemina.i18n.LocalizationManager;
import de.cnam.jemina.objects.Artikel;
import de.cnam.jemina.objects.Buch;
import de.cnam.jemina.resource.ImgThemeResource;
import de.cnam.jemina.ui.components.sub.DisplayBuch;
import org.apache.commons.lang3.StringUtils;

/**
 * Komponente um Artikel-Details anzuzeigen
 * @author CNAM Team (Nadine, Mike, Jens)
 */
public class DisplayArticle extends CustomComponent {
    
    // Fuer die Uebersetzung:
    private final LocalizationManager locale = LocalizationManager.getLocalizationManager(DisplayArticle.class);
    
    // UI
    private VerticalLayout infoLayout;
    private TextField id;
    private TextField name;
    private TextArea desc;
    private Button delete;
    private Button add;
    private Button save;
    private Button reserve;
    private Component details;
    
    // Daten
    private Artikel article;
    
    // Listener
    private IArticleControllerListener listener;
    
    // Berechtigungen
    private boolean canEdit;
    private boolean seeId;
    private boolean canDel;
    private boolean canAdd;
    private boolean canReserve;
    
    /**
     * Nur lesender Zugriff
     */
    public DisplayArticle() {
        this(false, false, false, false, false);
    }

    /**
     * Rechte k&ouml;nnen inizial gesetzt werden
     * @param canEdit Siehe {@link #setCanAdd(boolean) Setter-Beschreibung}
     * @param seeId Siehe {@link #setSeeId(boolean) Setter-Beschreibung}
     * @param canDel Siehe {@link #setCanDel(boolean) Setter-Beschreibung}
     * @param canAdd Siehe {@link #setCanAdd(boolean) Setter-Beschreibung}
     * @param canReserve Siehe {@link #setCanReserve(boolean) Setter-Beschreibung}
     */
    public DisplayArticle(boolean canEdit, boolean seeId, boolean canDel, boolean canAdd, boolean canReserve) {
        super();
        
        this.canEdit = canEdit;
        this.seeId = seeId;
        this.canDel = canDel;
        this.canAdd = canAdd;
        this.canReserve = canReserve;
        
        setCompositionRoot( genUi() );
        setStyleName("article");
    }
    
    private Component genUi() {
        HorizontalLayout layout = new HorizontalLayout();
        layout.setSizeFull();
        
        // Infos
        infoLayout = new VerticalLayout();
        infoLayout.setSizeFull();
        infoLayout.setSpacing(true);
        layout.addComponent(infoLayout);
        layout.setExpandRatio(infoLayout, 100F);
        
        id = new TextField(locale.localize("id"));
        name = new TextField(locale.localize("name"));
        desc = new TextArea(locale.localize("desc"));
        details = new Label();  // Dummy
        
//        id.setSizeFull();
//        name.setSizeFull();
        id.setWidth(100F, UNITS_PERCENTAGE);
        name.setWidth(100F, UNITS_PERCENTAGE);
        desc.setSizeFull();
        
        id.setReadOnly(true);
        name.setReadOnly(!canEdit);
        desc.setReadOnly(!canEdit);
        
        id.setVisible(seeId);
        
        infoLayout.addComponent(id);
        infoLayout.addComponent(name);
        infoLayout.addComponent(desc);
        infoLayout.addComponent(details);
        
        infoLayout.setExpandRatio(desc, 100F);
        
        // Aktionen:
        delete = new Button(locale.localize("delete"));
        add = new Button(locale.localize("add"));
        save = new Button(locale.localize("save"));
        reserve = new Button(locale.localize("reserve"));
        
        delete.setIcon(new ImgThemeResource("Garbage"));
        add.setIcon(new ImgThemeResource("Add"));
        save.setIcon(new ImgThemeResource("Save"));
        reserve.setIcon(new ImgThemeResource("Lock"));
        
        delete.setVisible(canDel);
        add.setVisible(canAdd);
        save.setVisible(canEdit);
        reserve.setVisible(canReserve);
        
        delete.setWidth(100F, UNITS_PERCENTAGE);
        add.setWidth(100F, UNITS_PERCENTAGE);
        save.setWidth(100F, UNITS_PERCENTAGE);
        reserve.setWidth(100F, UNITS_PERCENTAGE);
        
        delete.addStyleName("red");
        
//        delete.setHeight(60F, UNITS_PIXELS);
//        add.setHeight(60F, UNITS_PIXELS);
//        save.setHeight(60F, UNITS_PIXELS);
//        reserve.setHeight(60F, UNITS_PIXELS);
        
        // Layout Buttons
        VerticalLayout buttonLayout = new VerticalLayout();
        layout.addComponent(buttonLayout);
        buttonLayout.setHeight(100F, UNITS_PERCENTAGE);
        buttonLayout.setWidth(250F, UNITS_PIXELS);
        buttonLayout.setSpacing(true);
        buttonLayout.setMargin(true);
        buttonLayout.addComponent(reserve);
        buttonLayout.addComponent(save);
        buttonLayout.addComponent(add);
        buttonLayout.addComponent(delete);
        
        // Aktionen auf den Buttons
        delete.addListener(new Button.ClickListener() {
            @Override
            public void buttonClick(ClickEvent event) {
                if (listener != null) {
                    listener.deleteArticle(article);
                }
            }
        });
        
        add.addListener(new Button.ClickListener() {
            @Override
            public void buttonClick(ClickEvent event) {
                if (listener != null) {
                    listener.addExemplar(article);
                }
            }
        });
        
        save.addListener(new Button.ClickListener() {
            @Override
            public void buttonClick(ClickEvent event) {
                if (listener != null) {
                    listener.saveArticle(article);
                }
            }
        });
        
        reserve.addListener(new Button.ClickListener() {
            @Override
            public void buttonClick(ClickEvent event) {
                if (listener != null) {
                    listener.reserveArticle(article);
                }
            }
        });
        
        return layout;
    }
    
    /**
     * Dieser Listener wird informiert, wenn der Anwender einen Button geklickt hat
     * @param listener 
     */
    public void setActionListener(IArticleControllerListener listener) {
        this.listener = listener;
    }
    
    /**
     * Setzt den anzuzeigenden Artikel
     * @param article Artikel
     */
    public void setArticle(Artikel article) {
        this.article = article;
        
        // Zunaechst muss "ReadOnly" kurz aufgehoben werden
        id.setReadOnly(false);
        name.setReadOnly(false);
        desc.setReadOnly(false);
        
        // Werte setzen:
        if (article != null) {
            id.setValue(article.getId());
            name.setValue(article.getTitel());
            desc.setValue(article.getBeschreibung());
        } else {
            id.setValue(StringUtils.EMPTY);
            name.setValue(StringUtils.EMPTY);
            desc.setValue(StringUtils.EMPTY);
        }
        
        // ReadOnly wieder setzen:
        id.setReadOnly(true);
        name.setReadOnly(!canEdit);
        desc.setReadOnly(!canEdit);
        
        // Id anzeigen?
        id.setVisible(seeId);
        
        // Aktionen
        delete.setVisible(canDel);
        add.setVisible(canAdd);
        save.setVisible(canEdit);
        reserve.setVisible(canReserve);
        
        // Details
        if (article instanceof Buch) {
            infoLayout.removeComponent(details);
            details = new DisplayBuch((Buch)article,canEdit);
//            details.setHeight(150F, UNITS_PIXELS);
            infoLayout.addComponent(details);
        }
    }

    /**
     * Wenn <b>true</b> k&ouml;nnen die Artikel-Details ver&auml;ndert werden.
     * @param canEdit 
     */
    public void setCanEdit(boolean canEdit) {
        this.canEdit = canEdit;
    }

    /**
     * Wenn <b>true</b> wird die Artikel-ID angezeigt
     * @param seeId 
     */
    public void setSeeId(boolean seeId) {
        this.seeId = seeId;
    }

    /**
     * Wenn <b>true</b> kann der Artikel gel&ouml;scht werden
     * @param canDel 
     */
    public void setCanDel(boolean canDel) {
        this.canDel = canDel;
    }

    /**
     * Wenn <b>true</b> k&ouml;nnen Exemplare dem Artikel hinzugef&uuml;gt werden
     * @param canAdd 
     */
    public void setCanAdd(boolean canAdd) {
        this.canAdd = canAdd;
    }

    /**
     * Wenn <b>true</b> kann der Artikel reserviert werden
     * @param canReserve 
     */
    public void setCanReserve(boolean canReserve) {
        this.canReserve = canReserve;
    }
    
    
    /**
     * Listener f&uuml;r Aktionen die auf einem Artikel ausgef&uuml;hrt werden
     * k&ouml;nnen
     *
     * @author CNAM Team (Nadine, Mike, Jens)
     */
    public static interface IArticleControllerListener {

        /**
         * Der Anwender m&ouml;chte zu dem Artikel Exemplare anlegen
         *
         * @param a
         */
        void addExemplar(Artikel a);

        /**
         * Der Anwender m&ouml;chte den Artikel l&ouml;schen
         *
         * @param a
         */
        void deleteArticle(Artikel a);

        /**
         * Der Anwender m&ouml;chte den Artikel reservieren
         *
         * @param a
         */
        void reserveArticle(Artikel a);

        /**
         * Der Anwender m&ouml;chte die &Auml;nderungen an dem Artikel speichern
         *
         * @param a
         */
        void saveArticle(Artikel a);
    }
}
